//----------------------------------------------
// Forward rendering
//----------------------------------------------

#vertexshader

float4x4 matrixTotal;
float4x4 matrixWorldView;

struct VertexInput {
	float3 position : POSITION;
	float4 norm : NORMAL;		// compressed uint32
	float2 uv : TEXCOORD0;
	float2 uv2 : TEXCOORD1;
	float4 tangent : TEXCOORD2;	// compressed uint32
};

struct VertexOutput {
    float4 position : POSITION;
    float2 uv : TEXCOORD0;
    float2 uv2 : TEXCOORD1;
    float4 screenUV : TEXCOORD2;
    float4 rangeParams : TEXCOORD3;	// attenuation, fog, viewZ (for dof), etc.
};

VertexOutput main(VertexInput IN) {
	VertexOutput OUT;
	OUT.position =  mul(matrixTotal, float4(IN.position,1));
	OUT.uv = IN.uv;
	OUT.uv2 = IN.uv2;
	OUT.screenUV = OUT.position;
	OUT.rangeParams = mul(matrixWorldView, float4(IN.position, 1));	// view-space pos (viewZ for DOF)

	// decompress normal
/*	float3 N = 2.0f*IN.norm/255.0f-1.0f;

	// tangent-space
	OUT.nrm = mul(matrixWorldView, N);
	OUT.tan = mul(matrixWorldView, IN.tangent.xyz);
	OUT.bin = cross(OUT.nrm, OUT.tan)*IN.tangent.w;
*/
	return OUT;
};

#pixelshader

float4 paramsDOF;	// x=near, y=focus, z=far w=clamp_far

sampler tex1 : register(s0);
sampler tex2 : register(s1);
// s2 is normalmap
sampler lightBuffer : register(s3);

struct FpInput {
    float4 position : POSITION;
    float2 uv : TEXCOORD0;
    float2 uv2 : TEXCOORD1;
    float4 screenUV : TEXCOORD2;
    float4 rangeParams : TEXCOORD3;	// attenuation, fog, viewZ (for dof), etc.
};

// 'depth' is in view space
float computeDepthBlur(float depth) {
	float f;
	if(depth < paramsDOF.y) {
		f = (depth - paramsDOF.y) / (paramsDOF.y - paramsDOF.x);
	} else {
		f = (depth - paramsDOF.y) / (paramsDOF.z - paramsDOF.y);
		f = clamp(f, 0, paramsDOF.w);
	}
	return f * 0.5f + 0.5f;
}

float4 main(FpInput IN) : COLOR {
	float4 OUT;
	float2 sampleUV = (IN.screenUV.xy / IN.screenUV.w)* 0.5f + 0.5f;
	sampleUV.y = 1.0f - sampleUV.y;

	float4 light = tex2D(lightBuffer, sampleUV);
	float4 base = tex2D(tex1, IN.uv);
	float4 lightmap = tex2D(tex2, IN.uv2);
	OUT.rgb = 0.125 * (base.rgb * lightmap.rgb * 1.0f * light.rgb + 1.0f * light.aaa);
	OUT.a = computeDepthBlur( IN.rangeParams.z ); // compute DOF from viewZ
	return OUT;
};
