//----------------------------------------------
// Light
//----------------------------------------------
// NOTE: When filling the light buffer, be sure to sample the ZN buffer via point sampling!!!!

#vertexshader

struct VertexInput {
	float4 position : POSITION;
	float2 uv0 : TEXCOORD0;
};

struct VertexOutput {
    float4 position : POSITION;
    float2 uv0 : TEXCOORD0;
    float3 eyeToScreen : TEXCOORD1;
};

VertexOutput main(VertexInput IN) {
	VertexOutput OUT;
	OUT.position = IN.position;
	OUT.uv0 = IN.uv0;
	//                                aspect            tan(FOV/2), FOV=60
	//OUT.eyeToScreen = float3(IN.uv0.x * 1.0f, IN.uv0.y, 0.577350269f);
	OUT.eyeToScreen = float3(1, 1, 0.577350269f);
	return OUT;
};

#pixelshader

float4 lightPosition;		// in view-space
float4 lightColor;
sampler ZNBuffer : register(s0);

struct FpInput {
    float4 position : POSITION;
    float2 uv0 : TEXCOORD0;
    float3 eyeToScreen : TEXCOORD1;
};

/*
float DecodeFloat16(float2 v) {
	float h = v.x * 255.0f;
	float l = v.y * 255.0f;
	float f16 = h * 256.0f + l;
	float v2 = f16 / 65535.0f;
	return v2;
}*/

float DecodeFloat16(float2 v) {
	return (v.x * 4.0f + v.y) * 256.0f;
}

/*
float3 ConvertPolarToCartesian(float2 polar) {
	float2 sinPolar, cosPolar;
	sincos(polar, sinPolar, cosPolar);

	return float3(
		cosPolar.x * cosPolar.y,
		sinPolar.x * cosPolar.y,
		-sinPolar.y
	);
}*/

float4 main(FpInput IN) : COLOR {
	float4 OUT;
	float4 zn = tex2D(ZNBuffer, IN.uv0);
	float3 eyeToScreen = float3(IN.uv0.x*2.0f-1.0f * 1.0f, -(IN.uv0.y*2.0f-1.0f), 0.577350269f);

	float depth = DecodeFloat16(zn.zw); // * 1000.0f;
	float3 pixelPos = normalize(eyeToScreen) * depth;

	// extract view-space normal
	float3 N; // = ConvertPolarToCartesian(zn.xy); //*2-1);
	N.xy = zn.xy*2-1;
	N.z = -sqrt(1-dot(N.xy, N.xy));

	// L = vector to light
	float3 L = normalize(lightPosition.xyz-pixelPos);
	//float3 L = normalize(float3(0,0,1)-pixelPos);

	float NL = saturate(dot(L, N));
	float specular = pow(saturate(dot(reflect(normalize(float3(0.0, 0.0, 1.0)), N), L)), 22.0f);
//	float NV = 1.0f - saturate(pow(dot(L, N), 3.0f));

	OUT = float4(lightColor.x*NL, lightColor.y*NL, lightColor.z*NL, specular * NL);
//	OUT = float4(depth, depth, depth, 1.0f);
//	OUT = float4(pixelPos / 1000.0f, 1.0f);
//	OUT = float4(0, IN.uv0.y, 0, 1.0f);

	return OUT;
};
