#vertexshader

float4x4 matrixTotal;
float4x4 matrixWorldView;
float4x4 matrixWorldViewIT;

struct VertexInput {
	float4 position : POSITION;
	float4 norm : NORMAL;		// compressed uint32
	float2 uv : TEXCOORD0;
	float2 uv2 : TEXCOORD1;
	float4 tangent : TEXCOORD2;	// compressed uint32
};

struct VertexOutput {
    float4 position : POSITION;
    float2 uv : TEXCOORD0;
    float4 posInViewSpace : TEXCOORD1;
    float3 normalVS : TEXCOORD2;
    float3 tangentVS : TEXCOORD3;
    float3 binormalVS : TEXCOORD4;
};

VertexOutput main(VertexInput IN) {
	VertexOutput OUT;
	OUT.position =  mul(matrixTotal, IN.position);
	OUT.uv = IN.uv;
	OUT.posInViewSpace = mul(matrixWorldView, IN.position);

	// decompress normal & tangent
	float3 N = 2.0f*IN.norm/255.0f-1.0f;
	float4 T = 2.0f*IN.tangent/255.0f-1.0f;
	//float3 T = IN.tangent;

	// tangent-space
	OUT.normalVS = mul(matrixWorldViewIT, float4(N, 1));
	OUT.tangentVS = mul(matrixWorldViewIT, float4(T.xyz, 1));
//	OUT.normalVS = mul(transpose((float3x3)matrixWorldView), N);
//	OUT.tangentVS = mul(transpose((float3x3)matrixWorldView), T);
/*
	float3x3 worldviewrot = (float3x3)matrixWorldView;
	OUT.normalVS = mul(worldviewrot, N);
	OUT.tangentVS = mul(worldviewrot, T);
*/
//	OUT.binormalVS = cross(OUT.normalVS, OUT.tangentVS) * IN.tangent.w;
	OUT.binormalVS = cross(OUT.normalVS, OUT.tangentVS) * T.w;

//	OUT.nrmInViewSpace = mul(matrixWorldViewIT, float4(IN.tangent.xyz, 1));
//	OUT.nrmInViewSpace = IN.tangent.xyz;

//	OUT.nrmInViewSpace = mul(matrixWorldViewIT, float4(IN.norm, 1));
	return OUT;
};

#pixelshader

float4x4 matrixWorldViewIT;

sampler NormalMap : register(s2);

struct FpInput {
    float4 position : POSITION;
    float2 uv : TEXCOORD0;
    float4 posInViewSpace : TEXCOORD1;
    float3 normalVS : TEXCOORD2;
    float3 tangentVS : TEXCOORD3;
    float3 binormalVS : TEXCOORD4;
};

/*
float2 EncodeFloat16(float v) {
	float f = v * 65535.0f;
	float hi = f / 256.0f;
	float lo = f-(floor(hi)*256.0f);
	return float2( hi / 255.0f, lo / 255.0f );
}*/

float2 EncodeFloat16(float v) {
	float fac = v / 256.0f;
	float fra = frac(fac);
	return float2((fac-fra) / 4.0f, fra);
}

float2 ConvertCartesianToPolar(float3 cartesian) {
	static const float twoPi = 2.0f * 3.14159f;
	return float2(
		atan2(cartesian.x, cartesian.y),
		twoPi - atan2(length(cartesian.xy), cartesian.z)
	);
}

float4 main(FpInput IN) : COLOR {
	float4 OUT;

	float3 N = tex2D(NormalMap, IN.uv).xyz*2.0f-1.0f;
//N.y = -N.y;
	float3 normal = normalize(N.x*IN.tangentVS + N.y*IN.binormalVS + N.z*IN.normalVS);

//	float3 nrmInViewSpace = normalize(IN.nrmInViewSpace + mul(matrixWorldViewIT, N));
//	float3 nrmInViewSpace = mul(matrixWorldView, N.xyz);

	float depth = length(IN.posInViewSpace); // /1000.0f;

//	OUT.xy = IN.nrmInViewSpace.xy*0.5f+0.5f;		// view-space normal
	OUT.xy = normal.xy*0.5f+0.5f;				// view-space normal
//	OUT.xy = ConvertCartesianToPolar(normal*0.5f+0.5f);	// view-space normal
//	OUT.xy = float2(0,0);// ConvertCartesianToPolar(N*0.5f+0.5f);	// view-space normal

	OUT.zw = EncodeFloat16(depth);		// depth normalized to [0,1]
	return OUT;
}
