
#vertexshader

struct VsIn {
	float4 position: POSITION;
	float2 texCoord: TEXCOORD;
};
VsIn main(VsIn inp){
	return inp;
}

#pixelshader

float4 paramsDOF;	// xy = pixelSizeHigh, zw = pixelSizeLow
float4 paramsAUX;

sampler s0 : register(s0);	// original
sampler s1 : register(s1);	// blured
sampler s2 : register(s2);	// bloom/glare

static float2 poisson[8] = {
	float2( 0.0,      0.0),
	float2( 0.527837,-0.085868),
	float2(-0.040088, 0.536087),
	float2(-0.670445,-0.179949),
	float2(-0.419418,-0.616039),
	float2( 0.440453,-0.639399),
	float2(-0.757088, 0.349334),
	float2( 0.574619, 0.685879),
};

// maximum CoC radius and diameter in pixels
static float2 vMaxCoC = float2(5.0, 10.0);

// scale factor for maximum CoC size on low-res image
static float radiusScale = 0.4;

float4 PoissonDOFFilter(float2 texcoord)
{
	float4 cOut;
	float discRadius;
	float discRadiusLow;
	float centerDepth;

	cOut = tex2D(s0, texcoord); // fetch center tap
	centerDepth = cOut.a;		// save its depth

	// convert depth into blur radius in pixels
	discRadius = abs(cOut.a * vMaxCoC.y - vMaxCoC.x);

	// compute disc radius on low-res image
	discRadiusLow = discRadius * radiusScale;

	// re-use cOut as an accumulator
	cOut = 0;

	for (int t=0; t < 8; t++)
	{
		// fetch low-res tap
		float2 coordLow = texcoord + (paramsDOF.zw * poisson[t] * discRadiusLow);
		float4 tapLow = tex2D(s1, coordLow);

		// fetch high-res tap
		float2 coordHigh = texcoord + (paramsDOF.xy * poisson[t] * discRadius);
		float4 tapHigh = tex2D(s0, coordHigh);
	
		// put tap bluriness into [0,1] range
		float tapBlur = abs(tapHigh.a * 2.0 - 1.0);

		// mix low- and high-res taps based on tap bluriness
		float4 tap = lerp(tapHigh, tapLow, tapBlur);

		// apply leaking reduction: lower weight for taps that are
		// closer than the center tap and in focus
		tap.a = (tap.a >= centerDepth) ? 1.0 : abs(tap.a * 2.0 - 1.0);

		// accumulate
		cOut.rgb += tap.rgb * tap.a;
		cOut.a += tap.a;
	}

	// normalize and return result
	return (cOut / cOut.a);
}

float4 main(float2 uv0 : TEXCOORD0) : COLOR {
//	float4 original = tex2D(s0, uv0);
//	float4 blured = tex2D(s1, uv0);
	float4 glare = tex2D(s2, uv0);
//	float3 dof = lerp(original.rgb, blured, original.a);
	float4 col = PoissonDOFFilter(uv0) + glare; // * 0.5f;
//	float3 col = dof + blured;
//	float3 col = original.rgb + blured.rgb;
//	float3 col = lerp(original, blured, 0.4f);
//	float3 col = blured;

	float3 colScaled = paramsAUX.y / paramsAUX.x * col.rgb;	// targetLum / avgLum * col
	float3 colMapped = colScaled / (colScaled + 1.0f);	// tone map (compression)
	return float4(colMapped, 1.0f);
	//return float4(sqrt(colMapped), 1.0f);
};
